@@ -98,8 +98,8 @@ namespace xt
 
         container_type m_data;
 
-        container_type& data_impl();
-        const container_type& data_impl() const;
+        container_type& data_impl() noexcept;
+        const container_type& data_impl() const noexcept;
 
         friend class xcontainer<xarray<T>>;
     };
@@ -167,8 +167,8 @@ namespace xt
 
         container_type& m_data;
 
-        container_type& data_impl();
-        const container_type& data_impl() const;
+        container_type& data_impl() noexcept;
+        const container_type& data_impl() const noexcept;
 
         using temporary_type = typename xcontainer_inner_types<self_type>::temporary_type;
         void assign_temporary_impl(temporary_type& tmp);
@@ -355,13 +355,13 @@ namespace xt
     //@}
 
     template <class T>
-    inline auto xarray<T>::data_impl() -> container_type&
+    inline auto xarray<T>::data_impl() noexcept -> container_type&
     {
         return m_data;
     }
 
     template <class T>
-    inline auto xarray<T>::data_impl() const -> const container_type&
+    inline auto xarray<T>::data_impl() const noexcept -> const container_type&
     {
         return m_data;
     }
@@ -445,13 +445,13 @@ namespace xt
     //@}
 
     template <class C>
-    inline auto xarray_adaptor<C>::data_impl() -> container_type&
+    inline auto xarray_adaptor<C>::data_impl() noexcept -> container_type&
     {
         return m_data;
     }
 
     template <class C>
-    inline auto xarray_adaptor<C>::data_impl() const -> const container_type&
+    inline auto xarray_adaptor<C>::data_impl() const noexcept -> const container_type&
     {
         return m_data;
     }
@@ -465,7 +465,7 @@ namespace xt
         base_type::get_strides() = tmp.strides();
         base_type::get_backstrides() = tmp.backstrides();
         m_data.resize(tmp.size());
-        std::copy(tmp.data().begin(), tmp.data().end(), m_data.begin());
+        std::copy(tmp.data().cbegin(), tmp.data().cend(), m_data.begin());
     }
 }
 
