@@ -9,8 +9,11 @@
 #ifndef XTENSOR_HPP
 #define XTENSOR_HPP
 
-#include <array>
 #include <cstddef>
+#include <utility>
+#include <array>
+#include <vector>
+#include <algorithm>
 
 #include "xcontainer.hpp"
 #include "xsemantic.hpp"
@@ -90,8 +93,8 @@ namespace xt
 
         container_type m_data;
 
-        container_type& data_impl();
-        const container_type& data_impl() const;
+        container_type& data_impl() noexcept;
+        const container_type& data_impl() const noexcept;
 
         friend class xcontainer<xtensor<T, N>>;
     };
@@ -160,8 +163,8 @@ namespace xt
 
         container_type& m_data;
 
-        container_type& data_impl();
-        const container_type& data_impl() const;
+        container_type& data_impl() noexcept;
+        const container_type& data_impl() const noexcept;
 
         using temporary_type = typename xcontainer_inner_types<self_type>::temporary_type;
         void assign_temporary_impl(temporary_type& tmp);
@@ -270,13 +273,13 @@ namespace xt
     //@}
 
     template <class T, std::size_t N>
-    inline auto xtensor<T, N>::data_impl() -> container_type&
+    inline auto xtensor<T, N>::data_impl() noexcept -> container_type&
     {
         return m_data;
     }
 
     template <class T, std::size_t N>
-    inline auto xtensor<T, N>::data_impl() const -> const container_type&
+    inline auto xtensor<T, N>::data_impl() const noexcept -> const container_type&
     {
         return m_data;
     }
@@ -360,13 +363,13 @@ namespace xt
     //@}
 
     template <class C, std::size_t N>
-    inline auto xtensor_adaptor<C, N>::data_impl() -> container_type&
+    inline auto xtensor_adaptor<C, N>::data_impl() noexcept -> container_type&
     {
         return m_data;
     }
 
     template <class C, std::size_t N>
-    inline auto xtensor_adaptor<C, N>::data_impl() const -> const container_type&
+    inline auto xtensor_adaptor<C, N>::data_impl() const noexcept -> const container_type&
     {
         return m_data;
     }
@@ -374,13 +377,12 @@ namespace xt
     template <class C, std::size_t N>
     inline void xtensor_adaptor<C, N>::assign_temporary_impl(temporary_type& tmp)
     {
-        // TODO (performance improvement) : consider moving tmps
-        // shape and strides
+        // TODO (performance improvement) : consider moving tmps shape and strides
         base_type::get_shape() = tmp.shape();
         base_type::get_strides() = tmp.strides();
         base_type::get_backstrides() = tmp.backstrides();
         m_data.resize(tmp.size());
-        std::copy(tmp.data().begin(), tmp.data().end(), m_data.begin());
+        std::copy(tmp.data().cbegin(), tmp.data().cend(), m_data.begin());
     }
 }
 
